@@ -21,7 +21,7 @@ def _command_summary():
     from datalad.interface.base import get_interface_groups
     from datalad.interface.base import load_interface
 
-    groups = get_interface_groups(include_plugins=True)
+    groups = get_interface_groups()
     grp_short_descriptions = defaultdict(list)
     for group, _, specs in sorted(groups, key=lambda x: x[1]):
         for spec in specs:
@@ -38,20 +38,6 @@ def _command_summary():
 __doc__ += "\n\n{}".format(_command_summary())
 
 
-def _load_plugins():
-    from datalad.plugin import _get_plugins
-    from datalad.plugin import _load_plugin
-    import re
-
-    camel = re.compile(r'([a-z])([A-Z])')
-
-    for pname, props in _get_plugins():
-        pi = _load_plugin(props['file'], fail=False)
-        if pi is None:
-            continue
-        globals()[camel.sub('\\1_\\2', pi.__name__).lower()] = pi.__call__
-
-
 def _generate_extension_api():
     """Auto detect all available extensions and generate an API from them
     """
@@ -90,9 +76,7 @@ def _generate_extension_api():
 
 
 _generate_extension_api()
-_load_plugins()
 
 # Be nice and clean up the namespace properly
-del _load_plugins
 del _generate_extension_api
 del _command_summary
