@@ -30,6 +30,7 @@ from ..support.constraints import (
     EnsureNone,
     EnsureStr,
 )
+from datalad.support.exceptions import CapturedException
 from ..utils import (
     ensure_list,
 )
@@ -58,9 +59,9 @@ class CreateSiblingGitlab(Interface):
 
     An existing GitLab project, or a project created via the GitLab web
     interface can be configured as a sibling with the :command:`siblings`
-    command. Alternativly, this command can create a GitLab project at any
+    command. Alternatively, this command can create a GitLab project at any
     location/path a given user has appropriate permissions for. This is
-    particulary helpful for recursive sibling creation for subdatasets. API
+    particularly helpful for recursive sibling creation for subdatasets. API
     access and authentication are implemented via python-gitlab, and all its
     features are supported. A particular GitLab site must be configured in a
     named section of a python-gitlab.cfg file (see
@@ -237,13 +238,16 @@ class CreateSiblingGitlab(Interface):
         siteobjs = dict()
 
         # which datasets to process?
-        if path is None:
+        if path is None or ds.pathobj in path:
             for r in _proc_dataset(
                     ds, ds,
                     site, project, name, layout, existing, access,
                     dryrun, siteobjs, publish_depends, description):
                 yield r
-        if path or recursive:
+        # we need to find a subdataset when recursing, or when there is a path that
+        # could point to one, we have to exclude the parent dataset in this test
+        # to avoid undesired level-1 recursion into subdatasets
+        if any(p != ds.pathobj for p in (path or [])) or recursive:
             # also include any matching subdatasets
             for subds in ds.subdatasets(
                     path=path,
@@ -416,11 +420,13 @@ def _proc_dataset(refds, ds, site, project, remotename, layout, existing,
                 status='ok',
             )
         except Exception as e:
+            ce = CapturedException(e)
             yield dict(
                 res_kwargs,
                 # relay all attributes
                 status='error',
-                message=('Failed to create GitLab project: %s', exc_str(e))
+                message=('Failed to create GitLab project: %s', ce),
+                exception=ce
             )
             return
     else:
