@@ -12,7 +12,6 @@
 import os
 from os import chmod
 import stat
-import re
 import sys
 
 from os.path import join as opj, exists, basename
@@ -21,7 +20,7 @@ from ..dataset import Dataset
 from datalad.api import (
     create_sibling,
     install,
-    publish,
+    push,
 )
 from datalad.cmd import (
     WitlessRunner as Runner,
@@ -37,6 +36,7 @@ from datalad.tests.utils import (
     assert_dict_equal,
     assert_false,
     assert_in,
+    assert_in_results,
     assert_no_errors_logged,
     assert_not_equal,
     assert_not_in,
@@ -51,7 +51,6 @@ from datalad.tests.utils import (
     known_failure_windows,
     ok_,
     ok_endswith,
-    ok_exists,
     ok_file_has_content,
     ok_file_under_git,
     skip_if_on_windows,
@@ -78,42 +77,25 @@ import logging
 lgr = logging.getLogger('datalad.tests')
 
 
-def assert_publish_with_ui(target_path, rootds=False, flat=True):
+_have_webui = None
 
-    paths = [_path_(".git/hooks/post-update")]     # hooks enabled in all datasets
-    not_paths = []  # _path_(".git/datalad/metadata")]  # metadata only on publish
-                    # ATM we run post-update hook also upon create since it might
-                    # be a reconfiguration (TODO: I guess could be conditioned)
 
-    # web-interface html pushed to dataset root
-    web_paths = ['index.html', _path_(".git/datalad/web")]
-    if rootds:
-        paths += web_paths
-    # and not to subdatasets
-    elif not flat:
-        not_paths += web_paths
+def have_webui():
+    """Helper for a delayed import test to break circular imports from
+    deprecated extension, which gets its tests from the module too
+    """
+    global _have_webui
+    if _have_webui is not None:
+        return _have_webui
 
-    for path in paths:
-        ok_exists(opj(target_path, path))
-
-    for path in not_paths:
-        assert_false(exists(opj(target_path, path)))
-
-    hook_path = _path_(target_path, '.git/hooks/post-update')
-    # No longer the case -- we are no longer using absolute path in the
-    # script
-    # ok_file_has_content(hook_path,
-    #                     '.*\ndsdir="%s"\n.*' % target_path,
-    #                     re_=True,
-    #                     flags=re.DOTALL)
-    # No absolute path (so dataset could be moved) in the hook
-    with open(hook_path) as f:
-        assert_not_in(target_path, f.read())
-    # correct ls_json command in hook content (path wrapped in "quotes)
-    ok_file_has_content(hook_path,
-                        '.*datalad ls -a --json file \..*',
-                        re_=True,
-                        flags=re.DOTALL)
+    try:
+        import datalad_deprecated.sibling_webui
+        from datalad_deprecated.tests.test_create_sibling_webui \
+            import assert_publish_with_ui
+        _have_webui = True
+    except (ModuleNotFoundError, ImportError):
+        _have_webui = False
+    return _have_webui
 
 
 if lgr.getEffectiveLevel() > logging.DEBUG:
@@ -172,6 +154,10 @@ def test_invalid_call(path):
                 "sibling '%s' already configured (specify alternative name, or force reconfiguration via --existing",
                 'bogus'))
 
+    if not have_webui():
+        # need an extension package
+        assert_raises(RuntimeError, ds.create_sibling, '', ui=True)
+
 
 @slow  # 26sec on travis
 @skip_if_on_windows  # create_sibling incompatible with win servers
@@ -193,7 +179,7 @@ def test_target_ssh_simple(origin, src_path, target_rootpath):
             name="local_target",
             sshurl="ssh://datalad-test:{}".format(port),
             target_dir=target_path,
-            ui=True)
+            ui=have_webui())
         assert_not_in('enableremote local_target failed', cml.out)
 
     GitRepo(target_path, create=False)  # raises if not a git repo
@@ -243,7 +229,7 @@ def test_target_ssh_simple(origin, src_path, target_rootpath):
                 sshurl="ssh://datalad-test" + target_path,
                 publish_by_default=DEFAULT_BRANCH,
                 existing='replace',
-                ui=True,
+                ui=have_webui(),
             )
         interactive_assert_create_sshwebserver()
 
@@ -271,7 +257,7 @@ def test_target_ssh_simple(origin, src_path, target_rootpath):
             target_dir=target_path,
             target_url=target_path,
             target_pushurl="ssh://datalad-test" + target_path,
-            ui=True,
+            ui=have_webui(),
         )
 
         @with_testsui(responses=['yes'])
@@ -289,10 +275,13 @@ def test_target_ssh_simple(origin, src_path, target_rootpath):
         eq_("ssh://datalad-test" + target_path,
             source.repo.get_remote_url("local_target", push=True))
 
-        assert_publish_with_ui(target_path)
+        if have_webui():
+            from datalad_deprecated.tests.test_create_sibling_webui \
+                import assert_publish_with_ui
+            assert_publish_with_ui(target_path)
 
         # now, push should work:
-        publish(dataset=source, to="local_target")
+        push(dataset=source, to="local_target")
 
         # and we should be able to 'reconfigure'
         def process_digests_mtimes(digests, mtimes):
@@ -317,11 +306,16 @@ def test_target_ssh_simple(origin, src_path, target_rootpath):
                     digests.pop(f)
                     mtimes.pop(f)
             # and just pop some leftovers from annex
+            # and ignore .git/logs content (gh-5298)
             for f in list(digests):
-                if f.startswith('.git/annex/mergedrefs'):
+                if f.startswith('.git/annex/mergedrefs') \
+                        or f.startswith('.git/logs/'):
                     digests.pop(f)
                     mtimes.pop(f)
 
+        if not have_webui():
+            # the rest of the test assumed that we have uploaded a UI
+            return
         orig_digests, orig_mtimes = get_mtimes_and_digests(target_path)
         process_digests_mtimes(orig_digests, orig_mtimes)
 
@@ -381,7 +375,7 @@ def check_target_ssh_recursive(use_ssh, origin, src_path, target_path):
                 sshurl=sshurl,
                 target_dir=target_dir_tpl,
                 recursive=True,
-                ui=True)
+                ui=have_webui())
 
         # raise if git repos were not created
         for suffix in [sep + 'subm 1', sep + '2', '']:
@@ -389,13 +383,16 @@ def check_target_ssh_recursive(use_ssh, origin, src_path, target_path):
             # raise if git repos were not created
             GitRepo(target_dir, create=False)
 
-            assert_publish_with_ui(target_dir, rootds=not suffix, flat=flat)
+            if have_webui():
+                from datalad_deprecated.tests.test_create_sibling_webui \
+                    import assert_publish_with_ui
+                assert_publish_with_ui(target_dir, rootds=not suffix, flat=flat)
 
         for repo in [source.repo, sub1.repo, sub2.repo]:
             assert_not_in("local_target", repo.get_remotes())
 
         # now, push should work:
-        publish(dataset=source, to=remote_name)
+        push(dataset=source, to=remote_name)
 
         # verify that we can create-sibling which was created later and possibly
         # first published in super-dataset as an empty directory
@@ -405,23 +402,23 @@ def check_target_ssh_recursive(use_ssh, origin, src_path, target_path):
         # already
         with chpwd(source.path):
             # as we discussed in gh-1495 we use the last-published state of the base
-            # dataset as the indicator for modification detection with since=''
+            # dataset as the indicator for modification detection with since='^'
             # hence we must not publish the base dataset on its own without recursion,
             # if we want to have this mechanism do its job
-            #publish(to=remote_name)  # no recursion
+            #push(to=remote_name)  # no recursion
             assert_create_sshwebserver(
                 name=remote_name,
                 sshurl=sshurl,
                 target_dir=target_dir_tpl,
                 recursive=True,
                 existing='skip',
-                ui=True,
+                ui=have_webui(),
                 since=''
             )
-            assert_postupdate_hooks(target_path_, installed=True, flat=flat)
+            assert_postupdate_hooks(target_path_, installed=have_webui(), flat=flat)
         # so it was created on remote correctly and wasn't just skipped
         assert(Dataset(_path_(target_path_, ('prefix-' if flat else '') + sub3_name)).is_installed())
-        publish(dataset=source, to=remote_name, recursive=True, since='') # just a smoke test
+        push(dataset=source, to=remote_name, recursive=True, since='^') # just a smoke test
 
 
 @slow  # 28 + 19sec on travis
@@ -540,7 +537,7 @@ def check_replace_and_relative_sshpath(use_ssh, src_path, dst_path):
     create_tree(ds.path, {'sub.dat': 'lots of data'})
     ds.save('sub.dat')
     try:
-        ds.create_sibling(url, ui=True)
+        res = ds.create_sibling(url, ui=have_webui())
     except UnicodeDecodeError:
         if sys.version_info < (3, 7):
             # observed test failing on ubuntu 18.04 with python 3.6
@@ -548,15 +545,16 @@ def check_replace_and_relative_sshpath(use_ssh, src_path, dst_path):
             # We will just skip this tricky one
             raise SkipTest("Known failure")
         raise
-
-    published = ds.publish(to=sibname, transfer_data='all')
+    assert_in_results(res, action="create_sibling", sibling_name=sibname)
+    published = ds.push(to=sibname, data='anything')
     assert_result_count(published, 1, path=opj(ds.path, 'sub.dat'))
-    # verify that hook runs and there is nothing in stderr
-    # since it exits with 0 exit even if there was a problem
-    out = Runner(cwd=opj(dst_path, '.git')).run([_path_('hooks/post-update')],
-                                                protocol=StdOutErrCapture)
-    assert_false(out['stdout'])
-    assert_false(out['stderr'])
+    if have_webui():
+        # verify that hook runs and there is nothing in stderr
+        # since it exits with 0 exit even if there was a problem
+        out = Runner(cwd=opj(dst_path, '.git')).run([_path_('hooks/post-update')],
+                                                    protocol=StdOutErrCapture)
+        assert_false(out['stdout'])
+        assert_false(out['stderr'])
 
     # Verify that we could replace and publish no problem
     # https://github.com/datalad/datalad/issues/1656
@@ -569,29 +567,33 @@ def check_replace_and_relative_sshpath(use_ssh, src_path, dst_path):
     # for the test below depending on it
     with assert_raises(RuntimeError):
         # but we cannot replace in non-interactive mode
-        ds.create_sibling(url, existing='replace', ui=True)
+        ds.create_sibling(url, existing='replace', ui=have_webui())
 
     # We don't have context manager like @with_testsui, so
     @with_testsui(responses=["yes"])
     def interactive_create_sibling():
-        ds.create_sibling(url, existing='replace', ui=True)
+        ds.create_sibling(url, existing='replace', ui=have_webui())
     interactive_create_sibling()
 
-    published2 = ds.publish(to=sibname, transfer_data='all')
+    published2 = ds.push(to=sibname, data='anything')
     assert_result_count(published2, 1, path=opj(ds.path, 'sub.dat'))
 
     # and one more test since in above test it would not puke ATM but just
     # not even try to copy since it assumes that file is already there
     create_tree(ds.path, {'sub2.dat': 'more data'})
     ds.save('sub2.dat')
-    published3 = ds.publish(to=sibname, transfer_data='none')  # we publish just git
+    published3 = ds.push(to=sibname, data='nothing')  # we publish just git
     assert_result_count(published3, 0, path=opj(ds.path, 'sub2.dat'))
+
+    if not have_webui():
+        return
+
     # now publish "with" data, which should also trigger the hook!
     # https://github.com/datalad/datalad/issues/1658
     from glob import glob
     from datalad.consts import WEB_META_LOG
     logs_prior = glob(_path_(dst_path, WEB_META_LOG, '*'))
-    published4 = ds.publish(to=sibname, transfer_data='all')
+    published4 = ds.push(to=sibname, data='anything')
     assert_result_count(published4, 1, path=opj(ds.path, 'sub2.dat'))
     logs_post = glob(_path_(dst_path, WEB_META_LOG, '*'))
     eq_(len(logs_post), len(logs_prior) + 1)
@@ -703,12 +705,16 @@ def _test_target_ssh_inherit(standardgroup, ui, use_ssh, src_path, target_path):
 @slow  # 49 sec
 def test_target_ssh_inherit():
     skip_if_on_windows()  # create_sibling incompatible with win servers
+    try:
+        from datalad_deprecated.publish import Publish
+    except ImportError:
+        raise SkipTest('Test requires `publish()` from datalad-deprecated')
     # TODO: was waiting for resolution on
     #   https://github.com/datalad/datalad/issues/1274
     # which is now closed but this one is failing ATM, thus leaving as TODO
     # yield _test_target_ssh_inherit, None      # no wanted etc
     # Takes too long so one will do with UI and another one without
-    yield skip_ssh(_test_target_ssh_inherit), 'manual', True, True  # manual -- no load should be annex copied
+    yield skip_ssh(_test_target_ssh_inherit), 'manual', have_webui(), True  # manual -- no load should be annex copied
     yield _test_target_ssh_inherit, 'backup', False, False  # backup -- all data files
 
 
@@ -833,7 +839,7 @@ def test_non_master_branch(src_path, target_path):
     ds_a.create_sibling(
         name="sib", recursive=True,
         sshurl="ssh://datalad-test" + str(target_path / "b"))
-    ds_a.publish(to="sib", transfer_data="all")
+    ds_a.push(to="sib", data="anything")
 
     ds_b = Dataset(target_path / "b")
 
