@@ -115,10 +115,9 @@ class BaseDownloader(object, metaclass=ABCMeta):
         self.authenticator = authenticator
         self._cache = None  # for fetches, not downloads
         self._lock = InterProcessLock(
-            op.join(
-                cfg.obtain('datalad.locations.cache'),
-                'locks',
-                'downloader-auth.lck'))
+            op.join(cfg.obtain('datalad.locations.locks'),
+                    'downloader-auth.lck')
+        )
 
     def access(self, method, url, allow_old_session=True, **kwargs):
         """Generic decorator to manage access to the URL via some method
@@ -222,7 +221,7 @@ class BaseDownloader(object, metaclass=ABCMeta):
                     # give up
                     raise
                 lgr.debug("Failed to download fully, will try again: %s", exc_str(e))
-                # TODO: may be fail ealier than after 20 attempts in such a case?
+                # TODO: may be fail earlier than after 20 attempts in such a case?
             except DownloadError:
                 # TODO Handle some known ones, possibly allow for a few retries, otherwise just let it go!
                 raise
@@ -449,7 +448,7 @@ class BaseDownloader(object, metaclass=ABCMeta):
                 # eventually we might want to continue the download
                 lgr.warning(
                     "Temporary file %s from the previous download was found. "
-                    "It will be overriden" % temp_filepath)
+                    "It will be overridden" % temp_filepath)
                 # TODO.  also logic below would clean it up atm
 
             with open(temp_filepath, 'wb') as fp:
