@@ -47,12 +47,16 @@ def _test_consistent_order_of_args(intf, spec_posargs):
         # -- the message. But we don't enforce it on cmdline so it is
         # optional
         spec_posargs.add('message')
-    elif intf.__name__ == 'ExtractMetadata':
-        # MIH I was never sure what this test enforces and it takes
-        # me ages each time to try to wrap my head around it
-        # I am confident that I do not want to change the API of this
-        # command now that it is a command and no longer a plugin
-        # hence this exception
+    elif intf.__name__ in (
+            'AddReadme',
+            'Addurls',
+            'ExportArchive',
+            'ExportToFigshare',
+            'ExtractMetadata'):
+        if intf.__name__ != 'ExtractMetadata':
+            # ex-plugins had 'dataset' as the first positional argument
+            # and ExtractMetadata has 'types' as the first positional arg
+            eq_(args[0], 'dataset')
         eq_(spec_posargs, spec_posargs)
         return
 
