@@ -19,7 +19,6 @@ from ..support.network import get_local_file_url
 from ..support.external_versions import external_versions
 from ..utils import swallow_outputs
 from ..utils import swallow_logs
-from ..utils import on_windows
 
 from ..version import __version__
 from . import _TEMP_PATHS_GENERATED
@@ -27,18 +26,9 @@ from .utils import get_tempfile_kwargs
 from datalad.customremotes.base import init_datalad_remote
 from datalad import cfg as dl_cfg
 
-
-# we need a local file, that is supposed to be treated as a remote file via
-# file-scheme URL
-remote_file_fd, remote_file_path = \
-    tempfile.mkstemp(**get_tempfile_kwargs(
-        {'dir': dl_cfg.get("datalad.tests.temp.dir")}, prefix='testrepo'))
-# to be removed upon teardown
-_TEMP_PATHS_GENERATED.append(remote_file_path)
-with open(remote_file_path, "w") as f:
-    f.write("content to be annex-addurl'd")
-# OS-level descriptor needs to be closed!
-os.close(remote_file_fd)
+# eventually become a URL to a local file served via http
+# that can be used for http/url-based testing
+remote_file_url = None
 
 
 class TestRepo(object, metaclass=ABCMeta):
@@ -106,19 +96,18 @@ class BasicAnnexTestRepo(TestRepo):
     REPO_CLASS = AnnexRepo
 
     def populate(self):
+        global remote_file_url
+        if not remote_file_url:
+            # we need a local file, that is server via a URL
+            from datalad import test_http_server
+            remote_file_name = 'testrepo-annex.dat'
+            with open(opj(test_http_server.path, remote_file_name), "w") as f:
+                f.write("content to be annex-addurl'd")
+            remote_file_url = '{}/{}'.format(test_http_server.url, remote_file_name)
         self.create_info_file()
         self.create_file('test.dat', '123\n', annex=False)
         self.repo.commit("Adding a basic INFO file and rudimentary load file for annex testing")
-        # even this doesn't work on bloody Windows
-        fileurl = get_local_file_url(remote_file_path, compatibility='git-annex')
-        # Note:
-        # The line above used to be conditional:
-        # if not on_windows \
-        # else "https://raw.githubusercontent.com/datalad/testrepo--basic--r1/master/test.dat"
-        # This self-reference-ish construction (pointing to 'test.dat'
-        # and therefore have the same content in git and annex) is outdated and
-        # causes trouble especially in annex V6 repos.
-        self.repo.add_url_to_file("test-annex.dat", fileurl)
+        self.repo.add_url_to_file("test-annex.dat", remote_file_url)
         self.repo.commit("Adding a rudimentary git-annex load file")
         self.repo.drop("test-annex.dat")  # since available from URL
 
